// PROJECT  :OnBoardLED.S 
// PURPOSE  :First look at AVR Assembly LAnguage within the Arduino IDE 
// COURSE   :ICS4U
// AUTHOR   :C. D'Arcy
// DATE     :2021 04 10
// MCU      :DDBv6 (ATtyin84)
// STATUS   :Working
// REFERENCE:http://darcy.rsgc.on.ca/ACES/TEI4M/Assembly/AVR8AssemblyLanguageDDB.html#onBoardLED
#include  <avr/io.h>
#define     DDR    DDRB-0x20       //avr-as requires relative addressing 
#define     PORT   PORTB-0x20      //ditto
// The CONVENTIONAL Arduino C toolchain expects to see setup() and loop() functions
.global setup
setup:
  ldi r16,1<<PB2          ;define the state of (4 IO pins) of PORT B
  out DDR,r16             ;set the Port's IO directions
ret

.global loop
loop:
  rcall turnLEDOn         ;the onboard LED should appear lit
  rcall delay1s           ;admire for a duration
  rcall turnLEDOff        ;the onboard LED should appear unlit
  rcall delay1s           ;admire for a duration
ret

// sets the onboard LED's IO pin to HIGH
turnLEDOn:
  sbi PORT,PB2
ret

// sets the onboard LED's IO pin to LOW
turnLEDOff:
  cbi PORT,PB2
ret

// simply burn clock cycles
// see: http://darcy.rsgc.on.ca/ACES/TEI4M/AVRdelay.html
delay1s:
; Assembly code auto-generated
; by utility from Bret Mulvey
; Delay 8 000 000 cycles
; 1s at 8.0 MHz

    ldi  r18, 41
    ldi  r19, 150
    ldi  r20, 128
L1: dec  r20
    brne L1
    dec  r19
    brne L1
    dec  r18
    brne L1
ret

// Alternatively, override  Arduino C's toolchain's main.cpp file...
/*.global main
main:
  ldi r16,1<<PB2          ;define the state of (4 IO pins) of PORT B
  out DDR,r16             ;set them
again:
  rcall turnLEDOn
  rcall delay1s
  rcall turnLEDOff
  rcall delay1s
  rjmp  again
ret
*/
